import { Observer, Sender } from "./lib/observer.ts";
import { Lock, Ref } from "./lib/utils.ts";

const enabled = Deno.env.get("PROFILE") === "true";

export async function startup() {
	if (!enabled) return;
	await with_(() => {});
}

export function tip(name: string) {
	if (!enabled) return;
	const now = Temporal.Now.instant();
	with_((sender) => sender.tip(name, Number(now.epochNanoseconds)));
}

export function top(name: string) {
	if (!enabled) return;
	const now = Temporal.Now.instant();
	with_((sender) => sender.top(name, Number(now.epochNanoseconds)));
}

export function section<T>(name: string, procedure: () => T): T {
	tip(name);
	const result = procedure();
	if (result instanceof Promise) return result.finally(() => top(name)) as unknown as T;
	else top(name);
	return result;
}

export async function report() {
	if (!enabled) return;
	await with_((sender) => sender.log());
}

const sender = new Lock(new Ref(null as Sender | null));
async function with_(procedure: (sender: Sender) => unknown) {
	return await sender.with(async (guard) => {
		const ref = guard.get();
		if (ref.value === null) {
			const observer = await Observer.spawn();
			ref.value = new Sender(observer);
		}
		return await procedure(ref.value);
	});
}
