import { program } from "../../data/generated.ts";
import { wait_file } from "./utils.ts";
import { Msg } from "./types.ts";
import { writeAll } from "https://deno.land/std@0.224.0/io/write_all.ts";

export class Sender {
	observer;

	constructor(observer: Observer) {
		this.observer = observer;
	}

	tip(section: string, timestamp_ns: number) {
		this.observer.send([0, section, timestamp_ns]);
	}

	top(section: string, timestamp_ns: number) {
		this.observer.send([1, section, timestamp_ns]);
	}

	log() {
		this.observer.send([2]);
	}
}

export class Observer {
	process;
	socket;

	constructor(process: Deno.ChildProcess, socket: Deno.UnixConn) {
		this.process = process;
		this.socket = socket;
	}

	static async spawn() {
		const socket_path = `/tmp/profiterole-${Date.now()}.sock`;
		const args = [
			"run",
			`--allow-read=${socket_path}`,
			`--allow-write=${socket_path}`,
			"--unstable-temporal",
			"-",
			socket_path,
		];
		const process = new Deno.Command("deno", { args, stdin: "piped" }).spawn();
		const encoded = new TextEncoder().encode(program);
		const writer = process.stdin.getWriter();
		await writer.write(encoded);
		await writer.close();
		await writer.closed;
		// process.unref();
		// TODO : figure a way to have the child process survive after death of parent process.
		//        then detects parent process is dead and log automatically.

		await wait_file(socket_path);

		const socket = await Deno.connect({ transport: "unix", path: socket_path });
		socket.unref();
		return new Observer(process, socket);
	}

	async send(message: Msg) {
		const encoded = new TextEncoder().encode(JSON.stringify(message) + "\n");
		await writeAll(this.socket, encoded);
	}
}
